//Design and code by Peter Fankhauser
//		with code pieces (css and js) by Joe Golike @golike for Trulia.com (heatmaps)
//		and Jason Davis' word clouds

var browser = BrowserDetect;
var mwidth = 280;
var wordcloud, size = [ mwidth + 20, mwidth + 20 ], cloudid = null;
var maxterms = 75;
var minprob = 0.05;
var layout = 1;
var wsel = 1;
var weights;
var p = 2;

var state = {
	'c' : 'lobjson',
	'x0' : 0,
	'y0' : 0,
	'x1' : 1,
	'y1' : 1,
	'x2' : 0,
	'y2' : 0,
	'x3' : 0,
	'y3' : 0
};

if (isOldBrowser()) {
	$('#old_browser_msg').show();
	$('#wtf').hide();
	$('fieldset#state').addClass('ff3');
	$('#ie8_percents').addClass('ff3');
	$('#share2').addClass('ff3');
	$('#poweredby.old_browsers').show();
}

var jsondir = 'lobjson';

getState();
setState();
var mp = new Array();
initmp();

var buckets = 9, colorScheme = 'rbow1';
var rown = 15;
var minmax;
var fillColor;
var colorSchemes = {
	'rbow1' : [
			[ "#66C2A5", "#ABDDA4", "#E6F598", "#f6faaa", "#FEE08B", "#FDAE61",
					"#F46D43", "#D53E4F", "#9E0142" ],
			[ "#5E4FA2", "#3288BD", "#66C2A5", "#ABDDA4", "#FEE08B", "#FDAE61",
					"#F46D43", "#D53E4F", "#9E0142" ] ],
	'ryb' : [
			[ "#FFFFCC", "#FFEDA0", "#FED976", "#FEB24C", "#FD8D3C", "#FC4E2A",
					"#E31A1C", "#BD0026", "#800026" ],
			[ "#313695", "#4575B4", "#74ADD1", "#ABD9E9", "#FEE090", "#FDAE61",
					"#F46D43", "#D73027", "#A50026" ] ]
};

var colors = colorSchemes.rbow1;

var cqp = {};
d3.json(jsondir + '/cqp.json', function(json) {
	cqp = json;
});

d3.json(jsondir + '/minmax.json', function(json) {
	minmax = json;
	minf = Math.min(minmax[1].tfreq.min[wsel], minmax[0].tfreq.min[wsel]);
	maxf = Math.max(minmax[1].tfreq.max, minmax[0].tfreq.max);
	fillColor = d3.scale.quantile().domain([ Math.log(minf), Math.log(maxf) ])
			.range(colors[1]);
	for (var i = 0; i < 3; i++) {
		initMatrix(i, 0);
		loadCloud(i);
	}

	createKey('ckey0', 0, 0, minmax[0].cdist.min, minmax[0].cdist.max,
			linHeight);
	createKey('ckey2', 1, 0, minf, maxf, logHeight);
	createKey('ckey1', 0, 1, minmax[1].cdist.min, minmax[1].cdist.max,
			linHeight);
	createKey('ckey3', 1, 1, minf, maxf, logHeight);
	initChoiceListeners();
});

function initMatrix(n, c) {
	d3.select('#vis' + n).classed(colorScheme, true);
	d3.json(jsondir + '/' + mp[n].file, function(json) {
		mp[n].data = json;
		if (n == 0) {
			document.title = mp[0].data.title;
			if (mp[0].data.matrix.length == 1) {
				state.x1 = 0;
				state.y1 = 0;
				setState();
				initmp();
			}
		}
		createTiles(n, c);
		reColorTiles(n, true);
		initEventListeners(n);
	})
}

function initChoiceListeners() {
	// p-value choice listener
	$('input[name="pvalue"]').change(
			function() {

				minprob = $(this).val();
				var pid = $(this).attr('id');
				d3.selectAll('fieldset#pvalue label').classed('sel', false);
				d3.select('fieldset#pvalue label[for="' + pid + '"]').classed(
						'sel', true);
				for (var i = 0; i < 3; i++) {
					reColorTiles(i, false);
					loadCloud(i);
				}
			});

	// layout choice listener
	$('input[name="layout"]').change(
			function() {
				layout = $(this).val();
				var pid = $(this).attr('id');
				d3.selectAll('fieldset#layout label').classed('sel', false);
				d3.select('fieldset#layout label[for="' + pid + '"]').classed(
						'sel', true);
				for (var i = 0; i < 3; i++) {
					loadCloud(i);
				}
			});

	// colorscheme choice listener
	$('input[name="color"]').change(
			function() {
				var oldColorScheme = colorScheme;
				colorScheme = $(this).val();
				colors = colorSchemes[colorScheme];
				fillColor = d3.scale.quantile().domain(
						[ Math.log(minf), Math.log(maxf) ]).range(colors[1]);
				var pid = $(this).attr('id');
				d3.selectAll('fieldset#color label').classed('sel', false);
				d3.select('fieldset#color label[for="' + pid + '"]').classed(
						'sel', true);
				for (var i = 0; i < 3; i++) {
					d3.select('#vis' + i).classed(oldColorScheme, false);
					d3.select('#vis' + i).classed(colorScheme, true);
					reColorTiles(i, false);
					loadCloud(i);
				}
				reColorKey('ckey0', 0);
				reColorKey('ckey2', 1);
				reColorKey('ckey1', 0);
				reColorKey('ckey3', 1);
			});

}

function initEventListeners(n) {

	// headers mouseover events
	$('#tiles' + n + ' th').hover(function() {

		$(this).addClass('sel');

	}, function() {

		$(this).removeClass('sel');

	});

	// tiles mouseover events
	$('#tiles' + n + ' td').hover(function() {
		$(this).addClass('sel');
		addSel($(this), 'sel');
	}, function() {
		$(this).removeClass('sel');
		removeSel($(this), 'sel');
	});
	// tiles click events
	// TODO: clean up this mess
	$('#tiles' + n + ' td')
			.click(
					function() {
						var id = $(this).attr('id');
						var tmp = $(this).attr('id').split(/[a-z]/), number = parseInt(tmp[1]), day = parseInt(tmp[2]), hour = parseInt(tmp[3]);
						if (n > 0) {
							state['x' + (n + 1)] = day;
							state['y' + (n + 1)] = hour;
							var oldsel1 = $('#' + mp[n].sel1);
							oldsel1.removeClass('sel' + n);
							removeSel(oldsel1, 'sel1');
							$(this).addClass('sel' + n);
							addSel($(this), 'sel1');
							mp[n].sel1 = id;
							var ftmp = mp[n].tfile.split(/\//);
							mp[n].tfile = ftmp[0] + "/terms_"
									+ formatInteger(day, 3) + "_"
									+ formatInteger(hour, 3) + ".json";
							loadCloud(n);
						} else {
							state['x0'] = day;
							state['y0'] = hour;
							var oldsel2 = $('#' + mp[n].sel2);
							var oldsel1 = $('#' + mp[n].sel1);
							var tmp1 = oldsel1.attr('id').split(/[a-z]/), number1 = parseInt(tmp1[1]), day1 = parseInt(tmp1[2]), hour1 = parseInt(tmp1[3]);
							state['x1'] = day1;
							state['y1'] = hour1;
							removeSel(oldsel2, 'sel2');
							oldsel2.removeClass('sel2');
							removeSel(oldsel1, 'sel1');
							oldsel1.removeClass('sel1');
							oldsel1.addClass('sel2');
							addSel(oldsel1, 'sel2');
							addSel($(this), 'sel1');
							$(this).addClass('sel1');
							oldsel2.children().children().eq(1).text("");
							oldsel1.children().children().eq(1).text("2");
							mp[n].sel2 = oldsel1.attr('id');
							mp[n].sel1 = id;
							if (mp[n].sel1 == mp[n].sel2) {
								$(this).children().children().eq(1)
										.text("\xBD");
							} else {
								$(this).children().children().eq(1).text("1");
							}
							switchMatrix(day, hour);
						}
						setState();
					});
}

function loadCloud(n) {
	d3.json(jsondir + '/' + mp[n].tfile, function(json) {
		var termlist = json.terms;
		var taglist = new Array();
		var length = termlist.length;
		var d = n;
		if (d > 1) {
			d = 1;
		}
		var maxfreq = minmax[d].tfreq.max;
		var minfreq = minmax[d].tfreq.min[wsel];
		weights = {};
		var i = 0;
		var j = 0;
		while (i < maxterms && j < termlist.length) {
			// filter by pvalue if available
			if (termlist[j].length == 3 || termlist[j][3] <= minprob) {
				taglist.push({
					'text' : termlist[j][0],
					'size' : termlist[j][1]
				});
				weights[termlist[j][0]] = termlist[j][2];
				i++;
			}
			j++;
		}
		var fontSize = d3.scale.log().domain(
				[ minmax[d].tdist.min[wsel], minmax[d].tdist.max ]).range(
				[ 15, 60 ]);
		// var fontSize = d3.scale.linear().domain(
		// [ minmax[d].tdist.min[wsel], minmax[d].tdist.max ]).range(
		// [ 10, 70 ]);
		cloudid = '#cloud' + n;
		$(cloudid).empty();
		if (layout <= 2) {
			d3.layout.cloud().size(size).words(taglist).fontSize(function(d) {
				return fontSize(+d.size);
			}).spiral('archimedean')
			// .rotate(function() { return ~~(Math.random() * 2) * 90; })
			.rotate(function() {
				return 0;
			}).randomize(layout == 2 ? true : false).padding(3).on("end", draw)
					.start();
		} else {
			list(cloudid, taglist, fontSize);
		}
		var termtitle = json.title;
		$(cloudid).append(
				'<div style="text-align:center" id="title' + n + '">'
						+ termtitle + '</div>');
	});
}

function list(cloudid, taglist, fontSize) {
	var ret = "";
	var i = 0;
	var sorttags = taglist;
	if (layout == 4) {
		sorttags = taglist.sort(function(a, b) {
			if (a.text < b.text)
				return -1;
			if (a.text > b.text)
				return 1;
			return 0;
		});
	}
	while (i < maxterms && i < sorttags.length) {
		var color = fillColor(weights[sorttags[i].text] > 0 ? Math
				.log(weights[sorttags[i].text]) : 0);
		var span = '<span style="font-size:' + fontSize(sorttags[i].size)
				+ 'px;color:' + color + ';">' + sorttags[i].text + '</span>';
		ret += span + ' ';
		i++;
	}
	$(cloudid).append(
			'<div style="height:' + size[1] + 'px;width:' + size[0]
					+ 'px;overflow:auto;line-height:0.9;text-align:justify">'
					+ '<div>' + ret + '</div></div');
	$(cloudid).children().children().children("span").click(function() {
		openInNewTab(genURL(this));
	});
}

function draw(words) {
	wordcloud = d3.select(cloudid).append("svg").attr("width", size[0]).attr(
			"height", size[1]).append("g").attr("transform",
			"translate(" + (size[0] / 2) + "," + (size[1] / 2) + ")");
	wordcloud.selectAll("text").data(words).enter().append("text").style(
			"font-size", function(d) {
				return d.size + "px";
			}).style("fill", function(d) {
		return fillColor(weights[d.text] > 0 ? Math.log(weights[d.text]) : 0);
	}).attr("text-anchor", "middle").attr("transform", function(d) {
		return "translate(" + [ d.x, d.y ] + ")rotate(" + d.rotate + ")";
	}).text(function(d) {
		return d.text;
	}).on("click", function() {
		openInNewTab(genURL(this));
	}).style("opacity", 1e-6).transition().duration(1000).style("opacity", 1);
}

function reloadMatrix(n) {
	d3.json(jsondir + '/' + mp[n].file, function(json) {
		mp[n].data = json;
		reColorTiles(n, true);
	});
}

function switchMatrix(day, hour) {
	var auxfile = mp[1].file;
	mp[1].file = "dir_" + formatInteger(day, 3) + "_" + formatInteger(hour, 3)
			+ "/matrix.json";
	reloadMatrix(1);
	mp[2].file = auxfile;
	reloadMatrix(2);
	mp[0].tfile = "terms_" + formatInteger(day, 3) + "_"
			+ formatInteger(hour, 3) + ".json";
	loadCloud(0);
	var ftmp1 = mp[1].tfile.split(/\//);
	mp[1].tfile = "dir_" + formatInteger(day, 3) + "_" + formatInteger(hour, 3)
			+ "/" + ftmp1[1];
	loadCloud(1);
	var tmp = mp[0].sel2.split(/[a-z]/), n = parseInt(tmp[1]), d = parseInt(tmp[2]), h = parseInt(tmp[3]);
	var ftmp2 = mp[2].tfile.split(/\//);
	mp[2].tfile = "dir_" + formatInteger(d, 3) + "_" + formatInteger(h, 3)
			+ "/" + ftmp2[1];
	loadCloud(2);
}

/* ************************** */

function selectedType() {

	// return d3.select('input[name="type"]:checked').property('value'); // IE8
	// doesn't like this
	return $('input[name="type"]:checked').val();
}

/* ************************** */

function getCalcs(n) {

	var d = n;
	if (d > 1) {
		d = 1;
	}
	return {
		'min' : minmax[d].cdist.min * 0.99,
		'max' : minmax[d].cdist.max
	};
};

/* ************************** */

function reColorTiles(n, flip) {

	var calcs = getCalcs(n), range = [];

	for (var i = 1; i <= buckets; i++) {
		range.push(i);
	}

	var data = mp[n].data;

	$('#title' + n).text(mp[n].data.title);

	var bucket = d3.scale.quantize().domain(
			[ calcs.min, calcs.max > 0 ? calcs.max : 1 ]).range(range), side = d3
			.select('#tiles' + n).attr('class');

	if (flip) {
		if (side === 'front') {
			side = 'back';
		} else {
			side = 'front';
		}
	}

	for (var x = 0; x < data.matrix.length; x++) {
		for (var y = 0; y < data.matrix[x].length; y++) {
            var sel1 = '#n' + n + 'd' + x + 'h' + y + ' .tile';
			var sel = '#n' + n + 'd' + x + 'h' + y + ' .tile .' + side, val = data.matrix[x][y];

			// erase all previous bucket designations on this cell
			for (var i = 0; i <= buckets + 1; i++) {
				var cls = 'q' + i + '-' + buckets;
				d3.select(sel).classed(cls, false);
			}

			// set new bucket designation for this cell
			val = negIfNotSignificant(val,data,x,y);
			var cls = 'q'
					+ (val > 0 ? bucket(val) : (val < 0 ? buckets + 1 : 0))
					+ '-' + buckets;
			d3.select(sel).classed(cls, true);
		}
	}
	if (flip) {
		flipTiles(n);
	}
}

/* ************************** */

function flipTiles(n) {

	var oldSide = d3.select('#tiles' + n).attr('class'), newSide = '';

	if (oldSide == 'front') {
		newSide = 'back';
	} else {
		newSide = 'front';
	}

	var flipper = function(n, h, d, side) {
		return function() {
			var sel = '#n' + n + 'd' + d + 'h' + h + ' .tile', rotateY = 'rotateY(180deg)';

			if (side === 'back') {
				rotateY = 'rotateY(0deg)';
			}
			if (browser.browser === 'Safari' || browser.browser === 'Chrome') {
				d3.select(sel).style('-webkit-transform', rotateY);
			} else {
				d3.select(sel).select('.' + oldSide).classed('hidden', true);
				d3.select(sel).select('.' + newSide).classed('hidden', false);
			}

		};
	};

	data = mp[n].data;

	for (var x = 0; x < data.matrix.length; x++) {
		for (var y = 0; y < data.matrix[x].length; y++) {
			var side = d3.select('#tiles' + n).attr('class');
			setTimeout(flipper(n, x, y, side), (x * 30) + (y * 30)
					+ (Math.random() * 100));
		}
	}
	d3.select('#tiles' + n).attr('class', newSide);
}

/* ************************** */

function createTiles(n, s) {
	var data = mp[n].data;
	// var html = '<p style="text-align:center" id="title' + n + '">'
	// + data.title + '</p>' + '<table id="tiles' + n + '" class="front">';
	var html = '<table id="tiles' + n
			+ '" class="front"><tr><td><div>&nbsp;</div></td>'
			+ '<td id="title' + n + '"style="text-align:center" colspan="'
			+ data.matrix.length + '">' + data.title + '</td></tr>';
	var width = mwidth / data.matrix.length;
	var w = width - 2;
	var fs = width * 0.9;
	var w1 = width - 6;

	html += '<tr><th><div>&nbsp;</div></th>';
	for (var x = 0; x < data.matrix.length; x++) {
		html += '<th class="h" id="x' + n + 'h' + x + '" title="'
				+ data.xmover[x] + '">' + '<div style="width:' + w1
				+ 'px;max-width:' + w1 + 'px">' + data.xlabels[x] + '</div>'
				+ '</th>';
	}
	html += '</tr>';

	for (var y = 0; y < data.matrix.length; y++) {
		html += '<tr class="d">';
		html += '<th id="y' + n + 'd' + y + '" title="' + data.ymover[y] + '">'
				+ '<div style="text-align:right;width:40px;height:' + w1
				+ ';max-height:' + w1 + 'px">' + data.ylabels[y]
				+ '&nbsp;</div></th>';
		for (var x = 0; x < data.matrix.length; x++) {
			html += '<td id="n' + n + 'd' + y + 'h' + x + '" class="d' + y
					+ ' x' + x + '"><div class="tile" style="width:' + width
					+ 'px;height:' + width
					+ 'px"><div class="face front" style="width:' + w
					+ 'px;height:' + w
					+ 'px"></div><div class="face back" style="width:' + w
					+ 'px;height:' + w + 'px;font-size:' + fs
					+ 'px;line-height:' + w + 'px"></div></div></td>';
		}
		html += '</tr>';
	}

	html += '</table>';
	d3.select('#vis' + n).html(html);
	if (n == 0) {
		var node = $('#' + mp[n].sel1);
		node.addClass('sel1');
		addSel(node, 'sel1');
		if (mp[n].sel1 == mp[n].sel2) {
			node.children().children().eq(1).text("\xBD");
		} else {
			node.children().children().eq(1).text("1");
			node = $('#' + mp[n].sel2);
			node.addClass('sel2');
			addSel(node, 'sel2');
			node.children().children().eq(1).text("2");
		}
	} else if (n == 1) {
		var node = $('#' + mp[n].sel1);
		node.addClass('sel1');
		addSel(node, 'sel1');

	} else {
		var node = $('#' + mp[n].sel1);
		node.addClass('sel2');
		addSel($('#' + mp[n].sel1), 'sel1');
	}
}

function createKey(ckey, c, s, min, max, fheight) {
	var html = '<table style="padding:0px">';
	var min1 = min;
	var max1 = max;
	if (c == 1) {
		min1 *= 100;
		max1 *= 100;
	}
	for (var y = colors[c].length - 1; y >= 0; y--) {
		html += '<tr class="d">';
		var color = '<td><div class="tile" id="' + ckey + 'b' + y
				+ '" style="width:' + (mwidth / rown) + 'px;height:'
				+ fheight(y, colors[c].length) + 'px;background-color:'
				+ colors[c][y] + ';"></div></td>';
		var key = '<th style="vertical-align:middle"><div style="width:auto;height:'
				+ 10 + 'px;">';
		if (y == 0) {
			key = '<th style="vertical-align:bottom"><div style="width:auto;height:'
					+ 10 + 'px;">' + min1.toFixed(p);
		} else if (y == (colors[c].length - 1) / 2) {
			key = '<th style="vertical-align:middle"><div style="width:auto;height:'
					+ 10
					+ 'px;">'
					+ (min1 + (max1 - min1) / 2).toFixed(p)
					+ '</div></td>';
		} else if (y == (colors[c].length - 1)) {
			key = '<th style="vertical-align:top"><div style="width:auto;height:'
					+ 10 + 'px;">' + max1.toFixed(p);
		} else {
			key = '<th><div style="height:' + 10 + 'px">';
		}
		key += '</div></td>';
		if (s == 0) {
			html += key + '<td><div style="height:10px">&nbsp;</div></td>'
					+ color;
		} else {
			html += color + '<td><div style="height:10px">&nbsp;</div></td>'
					+ key;
		}
		html += '</tr>';
	}
	html += '</table>';
	d3.select('#' + ckey).html(html);
}

function reColorKey(ckey, c) {
	for (var y = colors[c].length - 1; y >= 0; y--) {
		$('#' + ckey + 'b' + y).css({
			"background-color" : colors[c][y]
		});
	}
}

/* ************************** */

function isOldBrowser() {

	var result = false;
	if (browser.browser === 'Explorer' && browser.version < 9) {
		result = true;
	} else if (browser.browser === 'Firefox' && browser.version < 4) {
		result = true;
	}

	// console.log(result);

	return result;
}

function formatInteger(x, n) {
	var result = "";
	var nstring = x.toString();
	for (var i = 0; i < n - nstring.length; i++) {
		result = result + "0";
	}
	return result + nstring;
}

function log(val, base) {
	return Math.log(val) / Math.log(base);
}

function linHeight(x, y) {
	return mwidth / y;
}

function logHeight(x, y) {
	return mwidth * (log(x + 2, y + 2) - log(x + 1, y + 2));
}

function addSel(obj, sel) {
	var tmp = obj.attr('id').split(/[a-z]/), n = parseInt(tmp[1]), d = parseInt(tmp[2]), h = parseInt(tmp[3]);
	// alert(n + " " + d + " " + h);
	$('#x' + n + 'h' + h).addClass(sel);
	$('#y' + n + 'd' + d).addClass(sel);
}

function removeSel(obj, sel) {
	var tmp = obj.attr('id').split(/[a-z]/), n = parseInt(tmp[1]), d = parseInt(tmp[2]), h = parseInt(tmp[3]);
	$('#x' + n + 'h' + h).removeClass(sel);
	$('#y' + n + 'd' + d).removeClass(sel);
}

function getState() {
	var hash;
	var q = document.URL.split('#')[1];
	if (q != undefined) {
		q = q.split('&');
		for (var i = 0; i < q.length; i++) {
			hash = q[i].split('=');
			if (hash[0] in state) {
				state[hash[0]] = hash[1];

			}
		}
	}
}

function setState() {
	var s = "";
	for ( var key in state) {
		s += key + "=" + state[key] + "&";
	}
	s = s.slice(0, -1);
	window.location.hash = s;
}

function initmp() {
	mp.push(new Array);
	mp.push(new Array);
	mp.push(new Array);
	mp[0].file = "matrix.json";
	jsondir = state.c;
	mp[0].sel1 = 'n0d' + state.x0 + 'h' + state.y0;
	mp[0].sel2 = 'n0d' + state.x1 + 'h' + state.y1;
	var sel1 = formatInteger(state.x0, 3) + '_' + formatInteger(state.y0, 3);
	mp[0].tfile = 'terms_' + sel1 + '.json';
	mp[1].file = 'dir_' + sel1 + '/matrix.json';
	mp[1].sel1 = 'n1d' + state.x2 + 'h' + state.y2;
	mp[1].tfile = 'dir_' + sel1 + '/terms_' + formatInteger(state.x2, 3) + '_'
			+ formatInteger(state.y2, 3) + '.json';
	var sel2 = formatInteger(state.x1, 3) + '_' + formatInteger(state.y1, 3);
	mp[2].file = 'dir_' + sel2 + '/matrix.json';
	mp[2].sel1 = 'n2d' + state.x3 + 'h' + state.y3;
	mp[2].tfile = 'dir_' + sel2 + '/terms_' + formatInteger(state.x3, 3) + '_'
			+ formatInteger(state.y3, 3) + '.json';
}

function openInNewTab(url) {
	var win = window.open(url, '_blank');
	win.focus();
}

function genURL(x) {
	var id = x.parentNode.parentNode.parentNode.id;
	if (id === "cloud0") {
		return (cqp.query0
				+ cqp[mp[0].data.xlabels[state.x0]]
				+ cqp.query1
				+ x.textContent
				+ (cqp["query2"] ? cqp.query2
						+ cqp[mp[0].data.xlabels[state.x0] + "Div"] + "%22"
						: (cqp["query3"] ? "%22+%25c" : cqp.query3a
								+ cqp.labels[mp[0].data.xlabels[state.x0]]
								+ "%22")) + cqp.query5);
	} else if (id === "cloud1") {
		return (cqp.query0
				+ cqp[mp[0].data.xlabels[state.x0]]
				+ cqp.query1
				+ x.textContent
				+ (cqp["query3"] ? cqp.query3
						+ mp[1].data.xlabels[state.x2]
						+ (cqp["query4"] ? cqp.query4
								+ cqp[mp[0].data.xlabels[state.x0] + "Div"]
								+ "%22" : "%22") : cqp.query3a
						+ cqp.labels[mp[0].data.xlabels[state.x0]]
						+ cqp.query3b
						+ cqp.labels[mp[1].data.xlabels[state.x2]] + "%22") + cqp.query5);
	} else if (id === "cloud2") {
		return (cqp.query0
				+ cqp[mp[0].data.xlabels[state.x1]]
				+ cqp.query1
				+ x.textContent
				+ (cqp["query3"] ? cqp.query3
						+ mp[2].data.xlabels[state.x3]
						+ (cqp["query4"] ? cqp.query4
								+ cqp[mp[0].data.xlabels[state.x1] + "Div"]
								+ "%22" : "%22") : cqp.query3a
						+ cqp.labels[mp[0].data.xlabels[state.x1]]
						+ cqp.query3b
						+ cqp.labels[mp[2].data.xlabels[state.x3]] + "%22") + cqp.query5);
	}
}

function negIfNotSignificant(val, data, x, y) {
	if (data.sigmatrix0 !== undefined) {
		var minNum = 2;
//		don't color tile if there are not at least minNum significant words
		if (minprob == 0.01) {
			return (data.sigmatrix0[x][y]>=minNum ? val: -1);
		}
		if (minprob == 0.05) {
			return (data.sigmatrix1[x][y]>=minNum ? val: -1);
		}
		if (minprob == 0.1) {
			return (data.sigmatrix2[x][y]>=minNum ? val: -1);
		}
		if (minprob == 0.25) {
			return (data.sigmatrix3[x][y]>=minNum ? val: -1);
		}
		return (data.sigmatrix4[x][y]>=minNum ? val: -1);
	}
    return val;
}
